/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.hama.graph;

import junit.framework.Assert;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;

public class DxALSErrorAggregator<M extends Writable> extends AbstractAggregator<M, Vertex<?, ?, ?>> {
	private static final Log LOG = LogFactory.getLog(DxALSErrorAggregator.class);
    double wasteErrorSum = 0;
    Vertex<?, ?, ?> v = null;	//to store the vertex info so that aggregateInternal can add times by the edges num

    public void aggregate(Vertex<?, ?, ?> vertex, M value) {  
      
      if (vertex != null && value != null) {
    	  v = vertex;
    	  wasteErrorSum += ((DoubleWritable)value).get();   	  
      } else if(vertex == null && value != null){	//called in parseMessages when master receive aggregated value from peers
    	  wasteErrorSum += ((DoubleWritable)value).get();
      } else{
    	  LOG.error("Null vertex and value!!");
      }
    }

    public void aggregate(Vertex<?, ?, ?>  vertex, M oldValue, M newValue) {
    	if(vertex != null && newValue != null){
    		v = vertex;
    		wasteErrorSum += ((DoubleWritable)newValue).get();
    	} else if(vertex == null && newValue != null){
    		wasteErrorSum += ((DoubleWritable)newValue).get();
    	} else{
      	  LOG.error("Null vertex and value!!");
        }		
    }
    
    public void aggregateInternal() {
    	Assert.assertNotNull(v);
    	//since the predict error is relative to each edge, times is the edge num instead of vertices num
    	addTimesAggregated(v.getEdges().size());
    }
    
    public M getValue() {
      //LOG.info("[DX] getvalue is " + wasteErrorSum/getTimesAggregated().get());
      return (M)(new DoubleWritable(wasteErrorSum));
    }
    
    public M finalizeAggregation() {
        return (M)(new DoubleWritable(wasteErrorSum / getTimesAggregated().get()));
      }
}
