/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.hama.graph;

import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Writable;
import org.apache.hama.bsp.AbstractRefEdge;

/**
 * (BSP-g) The ref-edge class
 */
public final class RefEdge<V extends Writable, E extends Writable, M extends Writable> 
  extends AbstractRefEdge<V, E, M> {

  public static enum RefEdgeType {
    INEDGE, OUTEDGE
  }
  
  private RefEdgeType edgeType;
  //private RefEdge<V, E, M> pair;
  
  //private Vertex<V, E, M> vertex; 
  private final E cost;
  private boolean awaked; // check, means the neighbor awaked the vertex in last iteration
  //private boolean active; // set, the neighbor awaked the vertex in this iteration

  public RefEdge(Vertex<V, E, M> vertex, E cost, RefEdgeType type) {
    this.edgeType = type;
    this.vertex = vertex;
    
    if (cost == null || cost instanceof NullWritable) {
      this.cost = null;
    } else {
      this.cost = cost;
    }
    awaked = false;
    active = false;
  }

  /*public void setPair(RefEdge<V, E, M> pair) {
    this.pair = pair;
  }
  
  public RefEdge<V, E, M> getPair() {
    return this.pair;
  }*/
  
  public boolean isAwake() {
    return awaked;
  }
  
  /*public void activate() {
    this.active = true;
  }*/
  
  public void commit() {
    this.awaked = this.active;
    this.active = false;
  }
  
  public V getVertexID() {
    return this.vertex.getVertexID();
  }

  public Vertex<V, E, M> getVertex() {
    return (Vertex <V, E, M>) vertex;
  }
  
  public E getValue() {
    return cost;
  }

  public boolean isInEdge() {
    return this.edgeType == RefEdgeType.INEDGE;
  }
  
  public boolean isOutEdge() {
    return this.edgeType == RefEdgeType.OUTEDGE;
  }
  
  @Override
  public String toString() {
    return (this.isInEdge() ? "[In-Edge]" : "[Out-Edge]") + this.getVertexID() + ":" + this.getValue();
  }
}
